# example 3, data from four-variate Gauss copula

# definitions
n <- 1E6 # number of original samples
C <- matrix(c(1, -0.344, 0.141, -0.486,
              -0.344, 1, 0.586, 0.244,
              0.141, 0.586, 1, -0.544,
              -0.486, 0.244, -0.544, 1), nrow = 4) # correlation matrix
x3c <- c(0.5,0.1,0.9); x4c <- c(0.5,0.3,0.3) # conditioning planes in x3- and x4-directions
nb <- 1E5 # number of bootstrap samples
np <- 200 # number of plot points
color <- colorRampPalette(c("#FFFFFF","#E6E680","#E6BF1A",
                            "#E68000","#FF4026","#993380",
                            "#4D26BF","#262680","#000000")) # color ramp for plots
pmax <- c(4,20,6) # maximum of p(x1,x2|x3,x4) for plotting

# Gauss copula density (R is correlation matrix)
dgausscopula <- function(x, R) {
   n <- nrow(x) # number of points
   p <- matrix(sapply(x, function (y) qnorm(y, 0, 1)), nrow = n)
   I <- diag(x = 1, nrow = n, ncol = n) # identity matrix
   return(1/sqrt(det(R)) * exp(-1/2 * apply(p * ((solve(R) - I) %*% p), 2, sum)))
}

# generation original samples from Gauss copula
A <- eigen(C); B <- diag(A$values); A <- A$vectors
xs <- matrix(rnorm(nrow(C)*n), nrow = nrow(C))
xs <- A %*% (sqrt(B) %*% xs)
for (j in 1:nrow(xs)) {
   xs[j,] <- sapply(xs[j,], function(x) {return(pnorm(x, 0, sqrt(C[j,j])))})
}
pairs(t(xs[,seq(1, n, length.out = 1e4)]), labels = c("x1","x2","x3","x4"), pch = ".")
d <- det.construct(xs, lb = 0, ub = 0, dtalim = 1E5) # det for bootstrap

# loop over conditioning planes
for (j in 1:length(x3c)) {
   # plot title
   strg <- paste("p(x1,x2|x3=", toString(x3c[j]),",x4=", toString(x4c[j]), ")", sep = "")
   # grid for plotting
   x1 <- ((1:np)-0.5)/np; x2 <- x1 # x1 & x2 grid
   xg <- rbind(rep(x1,np), rep(x2,each = np), x3c[j]*rep(1,np^2), x4c[j]*rep(1,np^2)) # grid
   
   # analytical reference of p(x1,x2|x3,x4)
   px12cx34a <- matrix(dgausscopula(xg, C) / 
                       dgausscopula(matrix(c(x3c[j],x4c[j]), nrow = 2), C[3:4,3:4]), nrow = np)
   filled.contour(x1, x2, px12cx34a, zlim = c(0,pmax[j]), color = color)
   title(main = paste(strg,"reference"), xlab = "x1", ylab = "x2")
   
   # p(x1,x2|x3,x4) from det estimator
   pxd <- matrix(det.query(d,xg), nrow = np) # p(x1,x2,x3,x4)
   px34d <- det.query(det.construct(xs[3:4,]), matrix(c(x3c[j],x4c[j]), nrow = 2)) # p(x3,x4)
   filled.contour(x1, x2, pxd/px34d, zlim = c(0,pmax[j]), color = color)
   title(main = paste(strg,"det"), xlab = "x1", ylab = "x2")
   
   # bootstrap conditional on x3 = x3c[j] and x4 = x4c[j]
   x12r <- det.rnd(nb, d, c(x3c[j],x4c[j]), c(3,4))
   # p(x1,x2|x3,x4) from bootstrap data
   px12cx34d <- matrix(det.query(det.construct(x12r[1:2,], lb = 0, ub = 0), xg[1:2,]), nrow = np)
   filled.contour(x1, x2, px12cx34d, zlim = c(0,pmax[j]), color = color)
   title(main = paste(strg,"bootstrap"), xlab = "x1", ylab = "x2")
}
